@@ -2,7 +2,7 @@
 # Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )
 
 ############################################################################
-#    Copyright (C) 2007-2009 by Martin Ošmera                              #
+#    Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 by Martin Ošmera     #
 #    martin.osmera@gmail.com                                               #
 #                                                                          #
 #    This program is free software; you can redistribute it and#or modify  #
@@ -21,6 +21,11 @@
 #    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
 ############################################################################
 
+# >>> File inclusion guard
+if { ! [ info exists _GRAPH_TCL ] } {
+set _GRAPH_TCL _
+# <<< File inclusion guard
+
 # --------------------------------------------------------------------------
 # DESCRIPTION
 # Graph panel in the bottom panel - shows states of ports
@@ -30,10 +35,14 @@ source "${::LIB_DIRNAME}/bottompanel/graph_wdg.tcl"	;# Graph widget
 
 class Graph {
 	## COMMON
+	 # Bool: The message: "Performance warning" was already displayed to the user
+	common performance_warning_already_shown	0
+	 # Bool: show performance warning when enabling external HW simulation
+	common show_sim_per_warn			${::CONFIG(SHOW_PALE_WARN)}
 
 	# Variables related to object initialization
 	private variable data_list			;# Teportary variable -- Configuration list
-	private variable gui_initialized	0	;# Bool: GUI created
+	private variable graph_gui_initialized	0	;# Bool: GUI created
 	private variable parent				;# Parent widget
 
 	private variable grid_mode		{b}	;# Current grid mode (one of {b n x y})
@@ -66,7 +75,7 @@ class Graph {
 
 
 	## Object constructor
-	constructor {} {	
+	constructor {} {
 		# Configure localy used ttk styles
 		ttk::style configure Graph_ActiveTab.TButton	\
 			-background {#AAAAFF}			\
@@ -88,7 +97,10 @@ class Graph {
 	public method PrepareGraph {Parent _data_list} {
 		set parent $Parent
 		set data_list $_data_list
-		set gui_initialized 0
+		set graph_gui_initialized 0
+
+		# Enable or disable PALE
+		$this pale_on_off [lindex $data_list 2]
 	}
 
 	## Inform this tab than it has became active
@@ -99,8 +111,12 @@ class Graph {
 	## Initialize graph
 	 # @return void
 	public method CreateGraphGUI {} {
-		if {$gui_initialized} {return}
-		set gui_initialized 1
+		if {$graph_gui_initialized} {return}
+		set graph_gui_initialized 1
+
+		if {${::DEBUG}} {
+			puts "CreateGraphGUI \[ENTER\]"
+		}
 
 		# Create panel frames
 		set top_bar [frame $parent.top_bar -bg {#CCCCCC}]	;# Buttons for switching pages
@@ -127,7 +143,7 @@ class Graph {
 			-command "$this graph_switch_grid_mode 1"	\
 		]
 		DynamicHelp::add $grid_button -text [mc "Change grid"]
-		setStatusTip -widget $grid_button -text [mc "Change grid morfology"]
+		setStatusTip -widget $grid_button -text [mc "Change grid morphology"]
 		pack $grid_button -anchor n
 		bind $grid_button <Button-1> "$this graph_switch_grid_mode 1; break"
 		bind $grid_button <Button-3> "$this graph_switch_grid_mode -1; break"
@@ -164,7 +180,7 @@ class Graph {
 		pack $clear_marks_button
 
 		# Create graphs
-		set pages_manager	[PagesManager $bottom_frame.pages_manager -background {#eeeeee}]
+		set pages_manager	[PagesManager $bottom_frame.pages_manager -background ${::COMMON_BG_COLOR}]
 		set nb_state_frame	[$pages_manager add {state}]
 		set nb_latches_frame	[$pages_manager add {latches}]
 		set nb_output_frame	[$pages_manager add {output}]
@@ -226,22 +242,41 @@ class Graph {
 
 		# Validate the loaded confiuration
 		foreach mark_flags {mark_flags_s mark_flags_l mark_flags_o} {
-			set mark_flags_data [subst "\$$mark_flags"]
-			if {
-				![regexp {^[01]+$} $mark_flags_data]
-					||
-				[string bytelength $mark_flags_data] != 170
-			} then {
-				puts stderr "Invalid graph mark flags -- discarded"
-				set $mark_flags [string repeat {0 } 170]
+			set mark_flags_data [subst -nocommands "\$$mark_flags"]
+			if {[string index $mark_flags_data 0] == {X}} {
+				set mark_flags_data [string range $mark_flags_data 1 end]
+				if {
+					[string length $mark_flags_data] != 43
+						||
+					![string is xdigit $mark_flags_data]
+				} then {
+					puts stderr "Invalid graph mark flags -- discarded"
+					set $mark_flags [string repeat {0 } 170]
+				} else {
+					set bin [::NumSystem::hex2bin $mark_flags_data]
+					set len [string length $bin]
+					if {$len < 170} {
+						set bin "[string repeat {0} [expr {170 - $len}]]$bin"
+					}
+					set $mark_flags [split $bin {}]
+				}
 			} else {
-				set $mark_flags [split $mark_flags_data {}]
+				if {
+					![regexp {^[01]+$} $mark_flags_data]
+						||
+					[string bytelength $mark_flags_data] != 170
+				} then {
+					puts stderr "Invalid graph mark flags -- discarded"
+					set $mark_flags [string repeat {0 } 170]
+				} else {
+					set $mark_flags [split $mark_flags_data {}]
+				}
 			}
 		}
 		if {
 			$magnification != {0}	&&	$magnification != {1}	&&
 			$magnification != {2}	&&	$magnification != {3}
-		} {
+		} then {
 			puts stderr "Invalid graph magnification level -- setting to default"
 			set magnification 0
 		}
@@ -252,7 +287,7 @@ class Graph {
 		if {
 			$grid_mode != {b}	&&	$grid_mode != {n}	&&
 			$grid_mode != {y}	&&	$grid_mode != {x}
-		} {
+		} then {
 			puts stderr "Invalid graph grid mode -- setting to 'y'"
 			set grid_mode {y}
 		}
@@ -295,7 +330,7 @@ class Graph {
 		set win_y [winfo pointery .]
 
 		# Create legend window
-		set win [toplevel .graph_help_win -class {Help} -bg {#EEEEEE}]
+		set win [toplevel .graph_help_win -class {Help} -bg ${::COMMON_BG_COLOR}]
 		set frame [frame $win.f -bg {#555555} -bd 0 -padx 1 -pady 1]
 		wm overrideredirect $win 1
 
@@ -349,7 +384,7 @@ class Graph {
 			$canvas create line $x 15 [expr {$x + 20}] 15 -fill {#000000} -width 2
 
 		 # {X}	Access to external memory
-		} {
+		} else {
 			$canvas create rectangle $x 20	\
 				[expr {$x + 20}] 15	\
 				-fill {#00FF00} -width 0 -outline {#00FF00}
@@ -358,7 +393,7 @@ class Graph {
 				-fill {#FF0000} -width 0 -outline {#FF0000}
 		}
 		incr x 20
-		 # {-}	Undeterminable state
+		 # {-}	Indeterminable state
 		$canvas create line $x 15	\
 			[expr {$x + 5}] 11	\
 			[expr {$x + 10}] 15	\
@@ -386,18 +421,18 @@ class Graph {
 		if {$nc_instead_of_X} {
 			set tmp_txt [mc "Not connected"]
 		 # {X}	Access to external memory
-		} {
+		} else {
 			set tmp_txt [mc "Access to external memory"]
 		}
 		$canvas create line 40 23	40 86	50 86		\
 			-fill {#000000} -arrow first -arrowshape {6 6 2}
 		$canvas create text 50 86 -fill {#000000} -anchor w	\
 			-text $tmp_txt -font $::smallfont
-		 # {-}	Undeterminable state
+		 # {-}	Indeterminable state
 		$canvas create line 60 23	60 72	70 72		\
 			-fill {#000000} -arrow first -arrowshape {6 6 2}
 		$canvas create text 70 72 -fill {#000000} -anchor w	\
-			-text [mc "Undeterminable state"] -font $::smallfont
+			-text [mc "Indeterminable state"] -font $::smallfont
 		 # {?}	No voltage
 		$canvas create line 80 23	80 58	90 58		\
 			-fill {#000000} -arrow first -arrowshape {6 6 2}
@@ -467,15 +502,15 @@ class Graph {
 	}
 
 	## Draw interrupt line
-	 # @parm String = {} - If "nohistory" the history of interrupt lines will not be modified
+	 # @parm String nh={} - If "nohistory" the history of interrupt lines will not be modified
 	 # @return void
-	public method graph_draw_interrupt_line args {
-		if {!$gui_initialized} {CreateGraphGUI}
+	public method graph_draw_interrupt_line {{nh {}}} {
+		if {!$graph_gui_initialized} {CreateGraphGUI}
 		create_all_graph_widgets
 
-		$graph_state	graph_draw_interrupt_line $args
-		$graph_latches	graph_draw_interrupt_line $args
-		$graph_output	graph_draw_interrupt_line $args
+		$graph_state	graph_draw_interrupt_line $nh
+		$graph_latches	graph_draw_interrupt_line $nh
+		$graph_output	graph_draw_interrupt_line $nh
 	}
 
 	## Draw new port states in the graph
@@ -483,7 +518,7 @@ class Graph {
 	 # @parm List values	- Values to display ...
 	 # @return void
 	public method graph_new_output_state {target values} {
-		if {!$gui_initialized} {CreateGraphGUI}
+		if {!$graph_gui_initialized} {CreateGraphGUI}
 		create_all_graph_widgets
 
 		switch -- $target {
@@ -522,6 +557,52 @@ class Graph {
 	public method graph_change_status_on {} {
 		set drawing_on [expr {!$drawing_on}]
 		graph_commit_state_on_off
+
+		# Show performance warning
+		if {$show_sim_per_warn && !$performance_warning_already_shown && $drawing_on} {
+			set performance_warning_already_shown 1
+			if {[winfo exists .performance_warning_dialog]} {
+				destroy .performance_warning_dialog
+			}
+			set dialog [toplevel .performance_warning_dialog]
+			set top_frame [frame $dialog.top]
+			pack [label $top_frame.img -image ::ICONS::32::messagebox_info] -side left -padx 5
+			pack [label $top_frame.txt -text [mc "You have just enabled simulation of external devices. Having this feature enabled causes serious reduction of simulator performance, the number of instructions executed per second in real time usually decreases by a factor of hundreds, maybe even thousands."] -wraplength 300 -justify left] -side left -fill both -padx 5
+
+			set bottom_frame [frame $dialog.bottom]
+			pack [checkbutton $bottom_frame.chb		\
+				-text [mc "Do not display again"]	\
+				-onvalue 0				\
+				-offvalue 1				\
+				-variable ::Graph::show_sim_per_warn	\
+			] -anchor e -pady 5
+			pack [ttk::button $bottom_frame.button_ok		\
+				-text [mc "Ok"] -compound left			\
+				-image ::ICONS::16::ok				\
+				-command "grab release $dialog; destroy $dialog"\
+			] -pady 10
+			bind $bottom_frame.button_ok <Return> "grab release $dialog; destroy $dialog"
+			bind $bottom_frame.button_ok <KP_Enter> "grab release $dialog; destroy $dialog"
+
+			# Pack window frames
+			pack $top_frame -expand 1 -padx 5 -pady 5
+			pack $bottom_frame -padx 5 -fill x
+
+			# Window manager options -- modal window
+			wm iconphoto $dialog ::ICONS::16::info
+			wm title $dialog [mc "Performance warning"]
+			wm resizable $dialog 0 0
+			wm transient $dialog .
+			catch {grab $dialog}
+			wm protocol $dialog WM_DELETE_WINDOW "
+				grab release $dialog
+				destroy $dialog
+			"
+			raise $dialog
+			update
+			focus -force $bottom_frame.button_ok
+			tkwait window $dialog
+		}
 	}
 
 	## Commit new ON/OFF state
@@ -544,13 +625,13 @@ class Graph {
 
 		# ON
 		if {$drawing_on} {
-			$start_stop_button configure -style GreenBg.TButton -text "ON"
+			$start_stop_button configure -style GreenBg.TButton -text [mc "ON"]
 			$grid_button		configure -state normal
 			$clear_marks_button	configure -state normal
 
 		# OFF
 		} else {
-			$start_stop_button configure -style RedBg.TButton -text "OFF"
+			$start_stop_button configure -style RedBg.TButton -text [mc "OFF"]
 
 			$zoom_in_button		configure -state disabled
 			$zoom_out_button	configure -state disabled
@@ -656,23 +737,17 @@ class Graph {
 	## Create GUI of all graphs
 	 # @return void
 	private method create_all_graph_widgets {} {
-		if {!$gui_initialized} {CreateGraphGUI}
+		if {!$graph_gui_initialized} {CreateGraphGUI}
 
-		if {!$graph_state_created} {
-			$graph_state	CreateGraphGUI
-		}
-		if {!$graph_latches_created} {
-			$graph_latches	CreateGraphGUI
-		}
-		if {!$graph_output_created} {
-			$graph_output	CreateGraphGUI
-		}
+		Graph_create_tab state
+		Graph_create_tab latches
+		Graph_create_tab output
 	}
 
 	## Get graph configuration values -- for project save
 	 # @return List - Configuration list
 	public method graph_get_config {} {
-		if {!$gui_initialized} {CreateGraphGUI}
+		if {!$graph_gui_initialized} {CreateGraphGUI}
 
 		create_all_graph_widgets
 
@@ -688,7 +763,7 @@ class Graph {
 	 # @parm Int bits - Number of steps to take back
 	 # @return void
 	public method graph_stepback {bits} {
-		if {!$gui_initialized} {CreateGraphGUI}
+		if {!$graph_gui_initialized} {CreateGraphGUI}
 		if {!$drawing_on} {return}
 
 		create_all_graph_widgets
@@ -712,3 +787,7 @@ class Graph {
 		}
 	}
 }
+
+# >>> File inclusion guard
+}
+# <<< File inclusion guard
