@@ -2,7 +2,7 @@
 # Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )
 
 ############################################################################
-#    Copyright (C) 2007-2009 by Martin Ošmera                              #
+#    Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 by Martin Ošmera     #
 #    martin.osmera@gmail.com                                               #
 #                                                                          #
 #    This program is free software; you can redistribute it and#or modify  #
@@ -21,20 +21,25 @@
 #    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
 ############################################################################
 
+# >>> File inclusion guard
+if { ! [ info exists _COMPILER_TCL ] } {
+set _COMPILER_TCL _
+# <<< File inclusion guard
+
 # --------------------------------------------------------------------------
 # 8051 COMPILER - BASE NAMESPACE
 # --------------------------------------------------------------------------
 
 # Include other parts
 source "${::LIB_DIRNAME}/compiler/codelisting.tcl"	;# Code listing creator
-source "${::LIB_DIRNAME}/compiler/assembler.tcl"	;# Assemler
+source "${::LIB_DIRNAME}/compiler/assembler.tcl"	;# Assembler
 source "${::LIB_DIRNAME}/compiler/disassembler.tcl"	;# Disassembler
 source "${::LIB_DIRNAME}/compiler/preprocessor.tcl"	;# Preprocessor
 source "${::LIB_DIRNAME}/compiler/compilerconsts.tcl"	;# Compiler constant definitons
 source "${::LIB_DIRNAME}/compiler/external_compiler.tcl";# External compiler interface
 
 namespace eval Compiler {
-	variable error_count	;# Number of errors occured during compilation
+	variable error_count	;# Number of errors occurred during compilation
 	variable warning_count	;# Number of warning reported during compilation
 
 	variable in_IDE	0	;# Bool: Running in IDE (I mean GUI)
@@ -46,10 +51,10 @@ namespace eval Compiler {
 	 # @parm String project_dir			- Project directory
 	 # @parm String current_dir			- Current working directory
 	 # @parm String input_file_name			- Name of input source code
-	 # @parm String input_file_extension = {}	- Extension of input file
+	 # @parm String input_file_extension={}		- Extension of input file
 	 # @return Bool - result
-	proc compile {project_dir current_dir input_file_name input_file_extension} {
-		variable error_count	;# Number of errors occured during compilation
+	proc compile {project_dir current_dir input_file_name {input_file_extension {}}} {
+		variable error_count	;# Number of errors occurred during compilation
 		variable warning_count	;# Number of warning reported during compilation
 
 		# Compiler settings to defaults
@@ -58,12 +63,12 @@ namespace eval Compiler {
 		# Adjust compiler settings
 		if {${::Compiler::Settings::_print} == 2} {
 			set ::Compiler::Settings::PRINT 0
-		} {
+		} else {
 			set ::Compiler::Settings::PRINT 1
 		}
 		if {${::Compiler::Settings::_object} == 2} {
 			set ::Compiler::Settings::OBJECT 0
-		} {
+		} else {
 			set ::Compiler::Settings::OBJECT 1
 		}
 
@@ -88,13 +93,13 @@ namespace eval Compiler {
 			set asm [open $file r]
 			set asm_data [read $asm]
 			close $asm
-		}]} {
+		}]} then {
 			if {${::Compiler::Settings::NOCOLOR}} {
-				${Settings::TEXT_OUPUT_COMMAND} [::Compiler::msgc {EN}][mc "Unable to open the specified file. (%s)" $file]
-				${Settings::TEXT_OUPUT_COMMAND} [::Compiler::msgc {EN}][mc "Compilation FAILED !"]
-			} {
-				${Settings::TEXT_OUPUT_COMMAND} [mc "Unable to open the specified file. (\033\[34;1m%s\033\[m)" $file]
-				${Settings::TEXT_OUPUT_COMMAND} [mc "\033\[31;1mCompilation FAILED !\033\[m"]
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} [::Compiler::msgc {EN}][mc "Unable to open the specified file. (%s)" $file]
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} [::Compiler::msgc {EN}][mc "Compilation FAILED !"]
+			} else {
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} [mc "Unable to open the specified file. (\033\[34;1m%s\033\[m)" $file]
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} [mc "\033\[31;1mCompilation FAILED !\033\[m"]
 			}
 			return 0
 		}
@@ -102,34 +107,34 @@ namespace eval Compiler {
 		# Initialize preprocessor
 		if {!${::Compiler::Settings::QUIET}} {
 			if {${::Compiler::Settings::NOCOLOR}} {
-				${Settings::TEXT_OUPUT_COMMAND}	\
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 					"\n\n[::Compiler::msgc {SN}][mc {Compiling file: %s}  $input_file_name$input_file_extension]"
-			} {
-				${Settings::TEXT_OUPUT_COMMAND}	\
+			} else {
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 					[mc "\n\nCompiling file: \033\[34;1m%s\033\[m" $input_file_name$input_file_extension]
 			}
-			${Settings::TEXT_OUPUT_COMMAND}	\
+			${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 				[mc "Initializing pre-processor ..."]
 		}
 		set precompiledCode [PreProcessor::compile $current_dir $file $asm_data]
 		set asm_data {}
-		incr error_count ${PreProcessor::error_count}
-		incr warning_count ${PreProcessor::warning_count}
-		if {${PreProcessor::error_count} > 0} {
+		incr error_count ${::PreProcessor::error_count}
+		incr warning_count ${::PreProcessor::warning_count}
+		if {${::PreProcessor::error_count} > 0} {
 			if {${::Compiler::Settings::NOCOLOR}} {
-				${Settings::TEXT_OUPUT_COMMAND} [::Compiler::msgc {EN}][mc "Pre-processing FAILED !"]
-			} {
-				${Settings::TEXT_OUPUT_COMMAND} [mc "\033\[31;1mPre-processing FAILED !\033\[m"]
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} [::Compiler::msgc {EN}][mc "Pre-processing FAILED !"]
+			} else {
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} [mc "\033\[31;1mPre-processing FAILED !\033\[m"]
 			}
 			report_status $current_dir $input_file_name
 			return 0
 		}
 
-		if {${Settings::ABORT_VARIABLE}} {return 0}
+		if {${::Compiler::Settings::ABORT_VARIABLE}} {return 0}
 
 		# Initialize Assembler
 		if {!${::Compiler::Settings::QUIET}} {
-			${Compiler::Settings::TEXT_OUPUT_COMMAND} [mc "Compiling ..."]
+			${::Compiler::Settings::TEXT_OUPUT_COMMAND} [mc "Compiling ..."]
 		}
 		assembler::compile					\
 			[md5::md5 -hex -file $file]			\
@@ -139,152 +144,152 @@ namespace eval Compiler {
 			${::PreProcessor::included_files}		\
 			$precompiledCode
 		set ::PreProcessor::included_files {}
-		incr error_count ${assembler::error_count}
-		if {${assembler::error_count} > 0} {
+		incr error_count ${::assembler::error_count}
+		if {${::assembler::error_count} > 0} {
 			if {${::Compiler::Settings::NOCOLOR}} {
-				${Settings::TEXT_OUPUT_COMMAND} [::Compiler::msgc {EN}][mc "Compilation FAILED !"]
-			} {
-				${Settings::TEXT_OUPUT_COMMAND} [mc "\033\[31;1mCompilation FAILED !\033\[m"]
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} [::Compiler::msgc {EN}][mc "Compilation FAILED !"]
+			} else {
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} [mc "\033\[31;1mCompilation FAILED !\033\[m"]
 			}
 			report_status $current_dir $input_file_name
 			return 0
 		}
 
-		if {${Settings::ABORT_VARIABLE}} {return 0}
+		if {${::Compiler::Settings::ABORT_VARIABLE}} {return 0}
 
 		# Write resulting object code
-		if {${Settings::OBJECT}} {
-			if {${Settings::OBJECT_FILE} != {}} {
-				set object_file ${Settings::OBJECT_FILE}
-			} {
+		if {${::Compiler::Settings::OBJECT}} {
+			if {${::Compiler::Settings::OBJECT_FILE} != {}} {
+				set object_file ${::Compiler::Settings::OBJECT_FILE}
+			} else {
 				set object_file $input_file_name
 				append object_file {.hex}
 			}
 			if {${::Compiler::Settings::NOCOLOR}} {
-				${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 					[mc "Creating IHEX8 ...\t\t\t-> \"%s\"" $object_file]
-			} {
-				${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+			} else {
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 				[mc "Creating IHEX8 ...\t\t\t-> \"\033\[34;1m%s\033\[m\"" $object_file]
 			}
 			makeBackupFile $current_dir $object_file
 			if {[catch {
-				set hex [open [file join $current_dir $object_file] w 420]
-			}]} {
+				set hex [open [file join $current_dir $object_file] w 0640]
+			}]} then {
 				if {${::Compiler::Settings::NOCOLOR}} {
-					${Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[::Compiler::msgc {EN}][mc "Error: Unable to open file \"%s\" for writing" [file join $current_dir $object_file]]
-					${Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[::Compiler::msgc {EN}][mc "Compilation FAILED !"]
-				} {
-					${Settings::TEXT_OUPUT_COMMAND}	\
+				} else {
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "\033\[31;1mError\033\[m: Unable to open file \"\033\[34;1m%s\033\[m\" for writing" [file join $current_dir $object_file]]
-					${Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "\033\[31;1mCompilation FAILED !\033\[m"]
 				}
 				report_status $current_dir $input_file_name
 				return 0
 			} else {
-				puts -nonewline $hex ${assembler::hex}
+				puts -nonewline $hex ${::assembler::hex}
 				close $hex
 			}
 		}
 
-		if {${Settings::ABORT_VARIABLE}} {return 0}
+		if {${::Compiler::Settings::ABORT_VARIABLE}} {return 0}
 
 		# Write resulting binary object code
-		if {${Settings::CREATE_BIN_FILE}} {
+		if {${::Compiler::Settings::CREATE_BIN_FILE}} {
 			if {!${::Compiler::Settings::QUIET}} {
 				if {${::Compiler::Settings::NOCOLOR}} {
-					${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "Creating object file ...\t\t-> \"%s\"" "${input_file_name}.bin"]
-				} {
-					${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+				} else {
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "Creating object file ...\t\t-> \"\033\[34;1m%s\033\[m\"" "${input_file_name}.bin"]
 				}
 			}
 
 			makeBackupFile $current_dir "${input_file_name}.bin"
 			if {[catch {
-				set bin [open [file join $current_dir $input_file_name.bin] w 420]
-			}]} {
+				set bin [open [file join $current_dir $input_file_name.bin] w 0640]
+			}]} then {
 				if {${::Compiler::Settings::NOCOLOR}} {
-					${Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[::Compiler::msgc {EN}][mc "Error: Unable to open file \"%s\" for writing" [file join $current_dir $input_file_name.bin]]
-					${Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[::Compiler::msgc {EN}][mc "Compilation FAILED !"]
-				} {
-					${Settings::TEXT_OUPUT_COMMAND}	\
+				} else {
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "Error: Unable to open file \"\033\[34;1m%s\033\[m\" for writing" [file join $current_dir "${input_file_name}.bin"]]
-					${Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "\033\[31;1mCompilation FAILED !\033\[m"]
 				}
 				report_status $current_dir $input_file_name
 				return 0
 			} else {
 				fconfigure $bin -translation binary
-				puts -nonewline $bin ${assembler::bin}
+				puts -nonewline $bin ${::assembler::bin}
 				close $bin
 			}
 			set bin_data {}
 		}
 		set hex_data {}
 
-		if {${Settings::ABORT_VARIABLE}} {return 0}
+		if {${::Compiler::Settings::ABORT_VARIABLE}} {return 0}
 
 		# Write simulator data file
-		if {${Settings::CREATE_SIM_FILE}} {
+		if {${::Compiler::Settings::CREATE_SIM_FILE}} {
 			if {!${::Compiler::Settings::QUIET}} {
 				if {${::Compiler::Settings::NOCOLOR}} {
-					${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "Creating assembler debug file ...\t-> \"%s\"" "${input_file_name}.adf"]
-				} {
-					${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+				} else {
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "Creating simulator data file ...\t-> \"\033\[34;1m%s\033\[m\"" "${input_file_name}.adf"]
 				}
 			}
 			makeBackupFile $current_dir "${input_file_name}.adf"
 			if {[catch {
-				set sim [open [file join $current_dir $input_file_name.adf] w 420]
-			}]} {
+				set sim [open [file join $current_dir $input_file_name.adf] w 0640]
+			}]} then {
 				if {${::Compiler::Settings::NOCOLOR}} {
-					${Settings::TEXT_OUPUT_COMMAND}	\
-						[::Compiler::msgc {EN}][mc "Error: Unable to open file \"%s]\" for writing" [file join $current_dir $input_file_name.adf]
-					${Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+						[::Compiler::msgc {EN}][mc "Error: Unable to open file \"%s]\" for writing" [file join $current_dir $input_file_name.adf]]
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[::Compiler::msgc {EN}][mc "Compilation FAILED !"]
-				} {
-					${Settings::TEXT_OUPUT_COMMAND}	\
+				} else {
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "\033\[31;1mError\033\[m: Unable to open file \"\033\[34;1m%s\033\[m\" for writing" [file join $current_dir $input_file_name.adf]]
-					${Settings::TEXT_OUPUT_COMMAND}	\
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 						[mc "\033\[31;1mCompilation FAILED !\033\[m"]
 				}
 				report_status $current_dir $input_file_name
 				return 0
-			} {
-				puts -nonewline $sim ${assembler::adf}
+			} else {
+				puts -nonewline $sim ${::assembler::adf}
 				close $sim
 			}
 		}
 
-		if {${Settings::ABORT_VARIABLE}} {return 0}
+		if {${::Compiler::Settings::ABORT_VARIABLE}} {return 0}
 
 		# Report final status
 		report_status $current_dir $input_file_name
 		if {!${::Compiler::Settings::QUIET}} {
 			if {${::Compiler::Settings::optim_ena}} {
 				if {${::Compiler::Settings::NOCOLOR}} {
-					${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
-						[mc "Number of optimalizations performed: %s" ${::PreProcessor::optims}]
-				} {
-					${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
-						[mc "Number of optimalizations performed: \033\[1m%s\033\[m" ${::PreProcessor::optims}]
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+						[mc "Number of optimization performed: %s" ${::PreProcessor::optims}]
+				} else {
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+						[mc "Number of optimization performed: \033\[1m%s\033\[m" ${::PreProcessor::optims}]
 				}
 			}
 			if {${::Compiler::Settings::NOCOLOR}} {
-				${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 					[::Compiler::msgc {SN}][mc "Compilation successful. (time: %s sec.)" [expr {[clock seconds] - $sec}]]
-			} {
-				${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+			} else {
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
 					[mc "\033\[32;1mCompilation successful.\033\[m (time: %s sec.)" [expr {[clock seconds] - $sec}]]
 			}
 		}
@@ -307,25 +312,25 @@ namespace eval Compiler {
 	 # @parm String input_file_name	- Name of input file
 	 # @return void
 	proc report_status {current_dir input_file_name} {
-		variable error_count	;# Number of errors occured during compilation
+		variable error_count	;# Number of errors occurred during compilation
 		variable warning_count	;# Number of warning reported during compilation
 
 		# Determinate name of code listing file
-		if {${Settings::PRINT_FILE} != {}} {
-			set print_file ${Settings::PRINT_FILE}
-		} {
+		if {${::Compiler::Settings::PRINT_FILE} != {}} {
+			set print_file ${::Compiler::Settings::PRINT_FILE}
+		} else {
 			set print_file $input_file_name
 			append print_file {.lst}
 		}
 
-		# Message "Creting code listing file"
-		if {!${::Compiler::Settings::QUIET}} {
+		# Message "Creating code listing file"
+		if {!${::Compiler::Settings::QUIET} && ${::Compiler::Settings::PRINT}} {
 			if {${::Compiler::Settings::NOCOLOR}} {
-				${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
-					[mc "Creting code listing file ...\t\t-> \"%s\"" $print_file]
-			} {
-				${Compiler::Settings::TEXT_OUPUT_COMMAND}	\
-					[mc "Creting code listing file ...\t\t-> \"\033\[34;1m%s\033\[m\"" $print_file]
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+					[mc "Creating code listing file ...\t\t-> \"%s\"" $print_file]
+			} else {
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+					[mc "Creating code listing file ...\t\t-> \"\033\[34;1m%s\033\[m\"" $print_file]
 			}
 		}
 
@@ -333,7 +338,7 @@ namespace eval Compiler {
 		if {!${::Compiler::Settings::QUIET}} {
 			if {$::TRANSLATION_LOADED} {
 				set text [mc "%s errors, %s warnings" $error_count $warning_count]
-			} {
+			} else {
 				set text "$error_count error"
 				if {$error_count != 1} {
 					append text "s"
@@ -344,30 +349,32 @@ namespace eval Compiler {
 				}
 			}
 			if {${::Compiler::Settings::NOCOLOR}} {
-				${Compiler::Settings::TEXT_OUPUT_COMMAND} $text
-			} {
-				${Compiler::Settings::TEXT_OUPUT_COMMAND} "\033\[1m$text\033\[m"
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} $text
+			} else {
+				${::Compiler::Settings::TEXT_OUPUT_COMMAND} "\033\[1m$text\033\[m"
 			}
 		}
 
 		# Write code listing file
-		makeBackupFile $current_dir $print_file
-		if {[catch {
-			set lst [open [file join $current_dir $print_file] w 420]
-		}]} {
-			if {${::Compiler::Settings::NOCOLOR}} {
-				${Settings::TEXT_OUPUT_COMMAND}	\
-					[::Compiler::msgc {EN}][mc "Error: Unable to open file \"%s\" for writing" [file join $current_dir $print_file]]
-				${Settings::TEXT_OUPUT_COMMAND} [mc "Compilation FAILED !"]
-			} {
-				${Settings::TEXT_OUPUT_COMMAND}	\
-					[::Compiler::msgc {EN}][mc "Error: Unable to open file \"\033\[34;1m%s\033\[m\" for writing" [file join $current_dir $print_file]]
-				${Settings::TEXT_OUPUT_COMMAND} [mc "\033\[31;1mCompilation FAILED !\033\[m"]
+		if {${::Compiler::Settings::PRINT}} {
+			makeBackupFile $current_dir $print_file
+			if {[catch {
+				set lst [open [file join $current_dir $print_file] w 0640]
+			}]} then {
+				if {${::Compiler::Settings::NOCOLOR}} {
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+						[::Compiler::msgc {EN}][mc "Error: Unable to open file \"%s\" for writing" [file join $current_dir $print_file]]
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND} [mc "Compilation FAILED !"]
+				} else {
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND}	\
+						[::Compiler::msgc {EN}][mc "Error: Unable to open file \"\033\[34;1m%s\033\[m\" for writing" [file join $current_dir $print_file]]
+					${::Compiler::Settings::TEXT_OUPUT_COMMAND} [mc "\033\[31;1mCompilation FAILED !\033\[m"]
+				}
+				return 0
+			} else {
+				puts -nonewline $lst [CodeListing::getListing]
+				close $lst
 			}
-			return 0
-		} else {
-			puts -nonewline $lst [CodeListing::getListing]
-			close $lst
 		}
 	}
 
@@ -388,7 +395,7 @@ namespace eval Compiler {
 
 	## Namespace containing compiler settings
 	namespace eval Settings {
-		## Peerhole optimalization enable flag
+		## Peephole optimization enable flag
 		variable optim_ena		0	;# Bool: 0 == disabled; 1 == enabled
 
 		## Memory limits
@@ -500,8 +507,9 @@ namespace eval Compiler {
 				} default {
 					_nomod_value	_paging_value	_pagelength_value	_pagewidth_value
 					_title_value	_date_value	_object_file		_print_file
-				} {
-					set $var [subst "\$$default"]
+				} \
+			{
+				set $var [subst -nocommands "\$$default"]
 			}
 
 			# Finalize
@@ -555,3 +563,7 @@ namespace eval Compiler {
 
 # Compiler settings to defaults
 Compiler::Settings::restoreDefaults
+
+# >>> File inclusion guard
+}
+# <<< File inclusion guard
