@@ -2,7 +2,7 @@
 # Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )
 
 ############################################################################
-#    Copyright (C) 2007-2009 by Martin Ošmera                              #
+#    Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 by Martin Ošmera     #
 #    martin.osmera@gmail.com                                               #
 #                                                                          #
 #    This program is free software; you can redistribute it and#or modify  #
@@ -21,6 +21,11 @@
 #    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
 ############################################################################
 
+# >>> File inclusion guard
+if { ! [ info exists _RIGHTPANEL_TCL ] } {
+set _RIGHTPANEL_TCL _
+# <<< File inclusion guard
+
 # --------------------------------------------------------------------------
 # DESCRIPTION
 # Implements Right Panel
@@ -42,8 +47,6 @@ class RightPanel {
 	inherit RegWatches InstructionDetails SubPrograms HwManager
 
 	## COMMON
-	 # Conter of instances
-	common count		0
 	 # Background color for selected rows -- light
 	common selection_color		{#EEFFDD}
 	 # Background color for selected rows -- dark
@@ -60,9 +63,9 @@ class RightPanel {
 			{button_cancel}	"Add/Remove editor bookmark to/from current line"}
 		{separator}
 		{command	{Previous}	{}	0	"rightPanel_bm_up"	{1uparrow}
-			"Goto previous bookmark"}
+			"Go to previous bookmark"}
 		{command	{Next}		{}	0	"rightPanel_bm_up"	{1downarrow}
-			"Goto next bookmark"}
+			"Go to next bookmark"}
 		{separator}
 		{command	{Remove all}	{}	0	"editor_procedure {} clear_all_bookmarks {}"
 			{editdelete}	"Remove all bookmarks from the editor"}
@@ -73,9 +76,9 @@ class RightPanel {
 			{button_cancel}	"Add/Remove editor breakpoint to/from current line"}
 		{separator}
 		{command	{Previous}	{}	0	"rightPanel_bp_up"	{1uparrow}
-			"Goto previous breakpoint"}
+			"Go to previous breakpoint"}
 		{command	{Next}		{}	0	"rightPanel_bp_up"	{1downarrow}
-			"Goto next breakpoint"}
+			"Go to next breakpoint"}
 		{separator}
 		{command	{Remove all}	{}	0	"editor_procedure {} clear_all_breakpoints {}"
 			{editdelete}	"Remove all breakpoints from the editor"}
@@ -95,8 +98,8 @@ class RightPanel {
 	private variable table_of_symbols	;# ID of tab "Table of symbols"
 	private variable obj_idx		;# Number of this object
 
-	private variable bookmarks_menu		;# ID of popup menu for "Bookmarks"
-	private variable breakpoints_menu	;# ID of popup menu for "Breakpoints"
+	private variable bookmarks_menu   {}	;# ID of popup menu for "Bookmarks"
+	private variable breakpoints_menu {}	;# ID of popup menu for "Breakpoints"
 	private variable symbols_menu		;# ID of popup menu for "Symbol list"
 
 	private variable bm_pagesManager	;# ID of pages manager for tab "Bookmarks"
@@ -148,8 +151,6 @@ class RightPanel {
 
 	## Object constructor
 	constructor {} {
-		incr count
-		set obj_idx $count
 	}
 
 	## Object destructor
@@ -167,7 +168,7 @@ class RightPanel {
 	 # @parm Widget ParentPane	- parent paned window
 	 # @parm String watches_file	- definition file for register watches
 	 # @return void
-	public method Initialize_rightPanel {notebookframe ParentPane watches_file} {
+	public method initialize_rightPanel {notebookframe ParentPane watches_file} {
 
 		# Object variables
 		set parentPane $ParentPane	;# Parent container (some frame)
@@ -175,10 +176,7 @@ class RightPanel {
 		set notebook_frame $notebookframe
 
 		## Create NoteBook
-		set notebook [NoteBook $notebook_frame.ntb_rightPanel	\
-			-side top  -bg {#EEEEEE}		\
-			-arcradius 4				\
-		]
+		set notebook [ModernNoteBook $notebook_frame.ntb_rightPanel]
 
 		# Register notebook status bar tips
 		notebook_Sbar_set {rightpanel} [list \
@@ -200,57 +198,57 @@ class RightPanel {
 
 		if {!${::Editor::editor_to_use}} {
 			# Tab "Bookmarks"
-			set bookmarks [$notebook insert end [mc "Bookmarks"]		\
-				-image ::ICONS::16::bookmark_toolbar			\
-				-raisecmd "$this rightPanel_set_active_page Bookmarks"	\
-				-helptext [mc "List of bookmarks in editor (Ctrl+6)"]	\
+			set bookmarks [$notebook insert end {Bookmarks}				\
+				-image ::ICONS::16::bookmark_toolbar				\
+				-raisecmd [list $this rightPanel_set_active_page Bookmarks]	\
+				-helptext [mc "List of bookmarks in editor (Ctrl+6)"]		\
 			]
 			# Tab "Breakpoints"
-			set breakpoints [$notebook insert end [mc "Breakpoints"]		\
+			set breakpoints [$notebook insert end {Breakpoints}			\
 				-image ::ICONS::16::flag					\
-				-raisecmd "$this rightPanel_set_active_page Breakpoints"	\
+				-raisecmd [list $this rightPanel_set_active_page Breakpoints]	\
 				-helptext [mc "List of breakpoints in editor (Ctrl+7)"]		\
 			]
 			# Tab "Symbols"
-			set table_of_symbols [$notebook insert end [mc "Symbols"]	\
-				-image ::ICONS::16::_blockdevice			\
-				-raisecmd "$this rightPanel_set_active_page Symbols"	\
-				-helptext [mc "Symbol List"]				\
+			set table_of_symbols [$notebook insert end {Symbols}			\
+				-image ::ICONS::16::_blockdevice				\
+				-raisecmd [list $this rightPanel_set_active_page Symbols]	\
+				-helptext [mc "Symbol List"]					\
 			]
 			# Tab "Instruction"
-			set instruction [$notebook insert end [mc "Instruction"]	\
-				-image ::ICONS::16::info				\
-				-raisecmd "$this rightPanel_set_active_page Instruction" \
-				-helptext [mc "Instruction details (Ctrl+8)"]		\
-				-createcmd [list $this CreateInstructionDetailsGUI]	\
+			set instruction [$notebook insert end {Instruction}			\
+				-image ::ICONS::16::info					\
+				-raisecmd [list $this rightPanel_set_active_page Instruction]	\
+				-helptext [mc "Instruction details (Ctrl+8)"]			\
+				-createcmd [list $this CreateInstructionDetailsGUI]		\
 			]
 		}
 		 # Tab "Watches"
-		set watches [$notebook insert end [mc "Watches"]		\
-			-image ::ICONS::16::player_playlist			\
-			-raisecmd "$this rightPanel_set_active_page Watches"	\
-			-helptext [mc "Register watches (Ctrl+9)"]		\
-			-createcmd [list $this CreateRegWatchesGUI]		\
+		set watches [$notebook insert end {Watches}				\
+			-image ::ICONS::16::player_playlist				\
+			-raisecmd [list $this rightPanel_set_active_page Watches]	\
+			-helptext [mc "Register watches (Ctrl+9)"]			\
+			-createcmd [list $this CreateRegWatchesGUI]			\
 		]
 		 # Tab "Subprograms"
-		set subprograms [$notebook insert end [mc "Subprograms"]	\
-			-image ::ICONS::16::queue				\
-			-raisecmd "$this rightPanel_set_active_page Subprograms"\
-			-helptext [mc "Active subprograms (Ctrl+0)"]		\
-			-createcmd [list $this CreateSubProgramsGUI]		\
+		set subprograms [$notebook insert end {Subprograms}			\
+			-image ::ICONS::16::queue					\
+			-raisecmd [list $this rightPanel_set_active_page Subprograms]	\
+			-helptext [mc "Active subprograms (Ctrl+0)"]			\
+			-createcmd [list $this CreateSubProgramsGUI]			\
 		]
 		 # Tab "Hardware manager"
-		set hwmanager [$notebook insert end [mc "Hardware"]		\
-			-image ::ICONS::16::kcmpci				\
-			-raisecmd "$this rightPanel_set_active_page Hardware"	\
-			-helptext [mc "Hardware manager"]			\
-			-createcmd [list $this CreateHwManagerGUI]		\
+		set hwmanager [$notebook insert end {Hardware}				\
+			-image ::ICONS::16::kcmpci					\
+			-raisecmd [list $this rightPanel_set_active_page Hardware]	\
+			-helptext [mc "Hardware manager"]				\
+			-createcmd [list $this CreateHwManagerGUI]			\
 		]
 
 		 # Tab "Hide"
-		$notebook insert end [mc "Hide"]		\
-			-image ::ICONS::16::2rightarrow		\
-			-raisecmd "$this right_panel_show_hide"	\
+		$notebook insert end {Hide}				\
+			-image ::ICONS::16::2rightarrow			\
+			-raisecmd [list $this right_panel_show_hide]	\
 			-helptext [mc "Hide the panel"]
 
 		# Prepare panel componenets but do not create GUI elements
@@ -262,7 +260,6 @@ class RightPanel {
 			PrepareInstructionDetails $instruction
 		}
 
-
 		## Create Button bar
 		 # Button "Show"
 		set button_bar [frame $notebook_frame.button_bar]
@@ -347,15 +344,15 @@ class RightPanel {
 
 		if {!${::Editor::editor_to_use}} {
 			# Pack pages managers
-			set bm_pagesManager [PagesManager $bookmarks.pgm_rightPanel_bm -background {#eeeeee}]
+			set bm_pagesManager [PagesManager $bookmarks.pgm_rightPanel_bm -background ${::COMMON_BG_COLOR}]
 			pack $bm_pagesManager -expand 1 -fill both
 			$bm_pagesManager compute_size
 
-			set bp_pagesManager [PagesManager $breakpoints.pgm_rightPanel_pm -background {#eeeeee}]
+			set bp_pagesManager [PagesManager $breakpoints.pgm_rightPanel_pm -background ${::COMMON_BG_COLOR}]
 			pack $bp_pagesManager -expand 1 -fill both
 			$bp_pagesManager compute_size
 
-			set sm_pagesManager [PagesManager $table_of_symbols.sm_pagesManager -background {#eeeeee}]
+			set sm_pagesManager [PagesManager $table_of_symbols.sm_pagesManager -background ${::COMMON_BG_COLOR}]
 			pack $sm_pagesManager -expand 1 -fill both
 			$sm_pagesManager compute_size
 
@@ -363,9 +360,9 @@ class RightPanel {
 			set bookmarks_menu	$notebook_frame.menu_rightPanel_bookmarks
 			set breakpoints_menu	$notebook_frame.menu_rightPanel_breakpoints
 			set symbols_menu	$notebook_frame.menu_rightPanel_symbols
-			menuFactory $BREAKPOINTMENU	$breakpoints_menu	0 "$this " 0 {}
-			menuFactory $BOOKMARKMENU	$bookmarks_menu		0 "$this " 0 {}
-			menuFactory $SYMBOLSKMENU	$symbols_menu		0 "$this " 0 {}
+			menuFactory $BREAKPOINTMENU	$breakpoints_menu	0 "$this " 0 {} [namespace current]
+			menuFactory $BOOKMARKMENU	$bookmarks_menu		0 "$this " 0 {} [namespace current]
+			menuFactory $SYMBOLSKMENU	$symbols_menu		0 "$this " 0 {} [namespace current]
 		}
 
 
@@ -379,7 +376,7 @@ class RightPanel {
 		if {$PanelVisible} {
 			# Show NoteBook
 			$parentPane paneconfigure $notebook_frame -minsize 295
-			pack $notebook -expand 1 -fill both
+			pack [$notebook get_nb] -expand 1 -fill both -padx 5 -pady 5
 			$parentPane configure -sashwidth 2
 
 			if {[catch {
@@ -387,15 +384,15 @@ class RightPanel {
 				if {
 					${::Editor::editor_to_use} &&
 					([lsearch {Bookmarks Breakpoints Instruction Symbols} $active_page] != -1)
-				} {
+				} then {
 					set active_page {Watches}
 					$notebook raise {Watches}
 				}
-			}]} {
+			}]} then {
 				set active_page {Watches}
 				$notebook raise {Watches}
 			}
-		} {
+		} else {
 			# Show button bar
 			$parentPane paneconfigure $notebook_frame -minsize 0
 			pack $button_bar -anchor nw
@@ -413,7 +410,7 @@ class RightPanel {
 	 # @return void
 	public method rightPanel_scroll args {
 		# Local variables
-		set what	[lindex $args 0]
+		set what	[lindex $args 0]	;# m == Bookmarks; p == Breakpoints; s == Symbols
 		set cmd		[lindex $args 1]	;# Scroll command (moveto, scroll and such)
 		set frac	[lindex $args 2]	;# Fraction where to move
 		set units	[lindex $args 3]	;# Units (optonal)
@@ -436,7 +433,7 @@ class RightPanel {
 		if {$units == {}} {
 			$lnb yview $cmd $frac
 			$txt yview $cmd $frac
-		} {
+		} else {
 			$lnb yview $cmd $frac $units
 			$txt yview $cmd $frac $units
 		}
@@ -470,7 +467,7 @@ class RightPanel {
 				if {[winfo ismapped $scrollbar]} {
 					pack forget $scrollbar
 				}
-			} {
+			} else {
 				if {![winfo ismapped $scrollbar]} {
 					pack $scrollbar -side right -fill y -after $txt
 				}
@@ -490,11 +487,11 @@ class RightPanel {
 	public method rightPanel_refresh_font_settings {for_all} {
 		if {${::Editor::editor_to_use}} {return}
 		if {$for_all} {
-			foreach widget $LIST_bookmarks_lineNumbers	 {
+			foreach widget $LIST_bookmarks_lineNumbers {
 				$widget configure -font ${Editor::defaultFont_bold}
 			}
 
-			foreach widget $LIST_breakpoints_lineNumbers	 {
+			foreach widget $LIST_breakpoints_lineNumbers {
 				$widget configure -font ${Editor::defaultFont_bold}
 			}
 
@@ -527,7 +524,7 @@ class RightPanel {
 				}
 				incr i
 			}
-		} {
+		} else {
 			$bookmarks_lineNumbers		configure -font ${Editor::defaultFont_bold}
 			$bookmarks_text			configure -font ${Editor::defaultFont_bold}
 			$breakpoints_lineNumbers	configure -font ${Editor::defaultFont_bold}
@@ -574,7 +571,7 @@ class RightPanel {
 	 # @return void
 	private method create_tags_in_symbol_list {} {
 		set tags_to_define [list tag_label tag_constant tag_normal tag_macro]
-		foreach tag_def [concat ${::ASMsyntaxHighlight::hightlight_tags} ${::CsyntaxHighlight::hightlight_tags}] {
+		foreach tag_def [concat ${::ASMsyntaxHighlight::highlight_tags} ${::CsyntaxHighlight::highlight_tags}] {
 			if {[lsearch -ascii -exact $tags_to_define [lindex $tag_def 0]] == -1} {
 				continue
 			}
@@ -591,13 +588,13 @@ class RightPanel {
 			# Fonr slant
 			if {$tag_def_item(3) == 1} {
 				set tag_def_item(3) italic
-			} {
+			} else {
 				set tag_def_item(3) roman
 			}
 			# Font weight
 			if {$tag_def_item(4) == 1} {
 				set tag_def_item(4) bold
-			} {
+			} else {
 				set tag_def_item(4) normal
 			}
 
@@ -667,7 +664,7 @@ class RightPanel {
 		pack $bm_up_button -side left
 		DynamicHelp::add $bm_up_button -text [mc "Move to previous bookmark"]
 		setStatusTip -widget $bm_up_button	\
-			-text [mc "Goto to line of previous bookmark"]
+			-text [mc "Go to to line of previous bookmark"]
 		# Button "Down"
 		set bm_down_button [ttk::button $button_frame.but_rightPanel_bm_down	\
 			-image ::ICONS::16::1downarrow					\
@@ -678,7 +675,7 @@ class RightPanel {
 		pack $bm_down_button -side left
 		DynamicHelp::add $bm_down_button -text [mc "Move to the next bookmark"]
 		setStatusTip -widget $bm_down_button	\
-			-text [mc "Goto to line of next bookmark"]
+			-text [mc "Go to to line of next bookmark"]
 		# Separator
 		pack [ttk::separator $button_frame.but_rightPanel_bm_sep	\
 			-orient vertical					\
@@ -773,7 +770,7 @@ class RightPanel {
 		pack $bp_up_button -side left
 		DynamicHelp::add $bp_up_button -text [mc "Move to previous breakpoint"]
 		setStatusTip -widget $bp_up_button	\
-			-text [mc "Goto to line of previous breakpoint"]
+			-text [mc "Go to to line of previous breakpoint"]
 		 # Button "Down"
 		set bp_down_button [ttk::button $button_frame.but_rightPanel_bp_down	\
 			-image ::ICONS::16::1downarrow					\
@@ -784,7 +781,7 @@ class RightPanel {
 		pack $bp_down_button -side left
 		DynamicHelp::add $bp_down_button -text [mc "Move to next breakpoint"]
 		setStatusTip -widget $bp_down_button	\
-			-text [mc "Goto to line of next breakpoint"]
+			-text [mc "Go to to line of next breakpoint"]
 		 # Separator
 		pack [ttk::separator $button_frame.but_rightPanel_bp_sep	\
 			-orient vertical					\
@@ -1021,8 +1018,8 @@ class RightPanel {
 
 		if {[winfo exists $breakpoints_menu]}	{destroy $breakpoints_menu}
 		if {[winfo exists $bookmarks_menu]}	{destroy $bookmarks_menu}
-		menuFactory $BREAKPOINTMENU	$breakpoints_menu	0 "$this " 0 {}
-		menuFactory $BOOKMARKMENU	$bookmarks_menu		0 "$this " 0 {}
+		menuFactory $BREAKPOINTMENU	$breakpoints_menu	0 "$this " 0 {} [namespace current]
+		menuFactory $BOOKMARKMENU	$bookmarks_menu		0 "$this " 0 {} [namespace current]
 	}
 
 	## Invoke bookmarks popup menu
@@ -1208,7 +1205,7 @@ class RightPanel {
 		# Change current line in the editor
 		set block_select 1
 		$this editor_procedure {} goto $lineNum
-		update idle
+		update idletasks
 		set block_select 0
 
 		return 1
@@ -1236,8 +1233,32 @@ class RightPanel {
 		return $lineNum
 	}
 
+	## If the given line contain symbol declaration then select it in the list
+	 # This function should be called after change on the line in the editor
+	 # @parm Int lineNum - line number
+	 # @return Bool - result
+	public method rightPanel_sm_select {lineNum} {
+		if {!$enabled || $block_select} {return}
+		if {![info exists sm_text]} {return}
+
+		# Unset selection in the list
+		$sm_text tag remove curLine 1.0 end
+
+		# Check for bookmark presence
+		set idx0 [lsearch -ascii -exact [$sm_lineNumbers get 1.0 end] $lineNum]
+		if {$idx0 == -1} {return 0}
+
+		# Select the line
+		incr idx0
+		set idx1 $idx0
+		incr idx1
+		$sm_text tag add curLine $idx0.0 $idx1.0
+		$sm_text see $idx0.0
+		return 1
+	}
+
 	## If the given line contain bookmark then select it in the list
-	 # This function should be called after change line in the editor
+	 # This function should be called after change on the line in the editor
 	 # @parm Int lineNum - line number
 	 # @return Bool - result
 	public method rightPanel_bm_select {lineNum} {
@@ -1259,7 +1280,7 @@ class RightPanel {
 	}
 
 	## If the given line contain bookmark then select it in the list
-	 # This function should be called after change line in the editor
+	 # This function should be called after change on the line in the editor
 	 # @parm Int lineNum - line number
 	 # @return Bool - result
 	public method rightPanel_bp_select {lineNum} {
@@ -1267,7 +1288,7 @@ class RightPanel {
 		if {![info exists breakpoints_text]} {return}
 
 		# Check for bookmark presence
-		set idx0 [lsearch [$breakpoints_lineNumbers get 1.0 end] $lineNum]
+		set idx0 [lsearch -ascii -exact [$breakpoints_lineNumbers get 1.0 end] $lineNum]
 		if {$idx0 == -1} {return 0}
 
 		# Select the line
@@ -1316,7 +1337,7 @@ class RightPanel {
 		if {$idx == {end}} {
 			set row [$target_widget index end]
 			set row [expr {int($row) - 2}]
-		} {
+		} else {
 			set row [expr {int($idx)}]
 		}
 		# Iterate over source text tags and add them to target widget
@@ -1363,7 +1384,7 @@ class RightPanel {
 
 		if {$idx == -1} {
 			set idx {end}
-		} {
+		} else {
 			append idx {.0}
 		}
 
@@ -1439,7 +1460,7 @@ class RightPanel {
 
 		if {$idx == -1} {
 			set idx {end}
-		} {
+		} else {
 			append idx {.0}
 		}
 
@@ -1690,7 +1711,7 @@ class RightPanel {
 		if {$start_idx == {}} {
 			if {[lindex $lineNumbers end] != $end_line} {
 				return
-			} {
+			} else {
 				set start_idx 1.0
 			}
 		}
@@ -1764,7 +1785,7 @@ class RightPanel {
 		if {$start_idx == {}} {
 			if {[lindex $lineNumbers end] != $end_line} {
 				return
-			} {
+			} else {
 				set start_idx 1.0
 			}
 		}
@@ -1842,7 +1863,7 @@ class RightPanel {
 			return 1
 
 		# Not empty string
-		} {
+		} else {
 			$clr_b configure -state normal
 		}
 
@@ -1850,15 +1871,16 @@ class RightPanel {
 		set content [string toupper $content]
 		set e [expr {int([$sm_text index insert])}]
 		for {set i 1} {$i < $e} {incr i} {
-			if {![string first $content [string toupper [$sm_text get $i.0 [list $i.0 lineend]]]]} {
+			if {![string first $content [string toupper [$sm_text get $i.2 [list $i.0 lineend]]]]} {
 				$widget configure -style StringFound.TEntry
 
 				$sm_text tag remove curLine 1.0 end
-				$sm_text tag add curLine [list $i.0 linestart] [list $i.0+1l linestart]
+				$sm_text tag add curLine $i.0 [list $i.0+1l linestart]
+				$sm_text see $i.0
 
 				set block_select 1
-				$this editor_procedure {} goto $i
-				update idle
+				$this editor_procedure {} goto [$sm_lineNumbers get $i.0 [list $i.0 lineend]]
+				update idletasks
 				set block_select 0
 				set search_val_in_progress 0
 				return 1
@@ -1880,7 +1902,15 @@ class RightPanel {
 	 #	3 - Macro
 	 # @parm Bool add__remove	- 1 == Add; 0 == Remove
 	 # @return void
-	public method rightPanel_adjust_symbol_list {lineNum symbol_name symbol_type add__remove} {
+	public method rightPanel_adjust_symbol_list {lineNum symbol_name symbol_type add__remove {editor_object {}}} {
+		if {[$this is_splitted] && $editor_object != {}} {
+			set idx [lsearch -ascii -exact [$this cget -editors] $editor_object]
+
+			if {$idx != $current_editor_idx} {
+				rightPanel_switch_editor_vars $idx
+			}
+		}
+	
 		$sm_lineNumbers configure -state normal
 		$sm_text configure -state normal
 
@@ -1900,47 +1930,59 @@ class RightPanel {
 
 			if {$idx == -1} {
 				set idx [$sm_text index {end-1l}]
-			} {
+			} else {
 				append idx {.0}
 			}
 
-			$sm_lineNumbers insert $idx "$lineNum\n"
-			$sm_text insert $idx "$symbol_name\n"
-
 			switch -- $symbol_type {
 				0 {	;# Label
 					set tag {tag_label}
+					set icon {symbol1}
 				}
 				1 {	;# Constant
 					set tag {tag_constant}
+					set icon {symbol3}
 				}
 				2 {	;# C variable
 					set tag {tag_normal}
+					set icon {symbol4}
 				}
 				3 {	;# Macro
 					set tag {tag_macro}
+					set icon {symbol2}
 				}
 				7 {	;# C function
 					set tag {tag_c_func}
+					set icon {symbol0}
 				}
 				default {
 					set tag {}
+					set icon {symbol5}
 				}
 			}
 
+			$sm_lineNumbers insert $idx "$lineNum\n"
+
+			if {${::Editor::defaultCharHeight} < 16} {
+				$sm_text insert ${idx} { }
+			} else {
+				$sm_text image create $idx -image ::ICONS::16::$icon
+			}
+			$sm_text insert ${idx}+1c " $symbol_name\n"
+
 			if {$tag != {}} {
 				$sm_text tag add $tag $idx [list $idx lineend]
 			}
 
 		# Remove symbol
-		} {
+		} else {
 			if {$lineNum == {all}} {
 				set idx [list]
 				set e [expr {int([$sm_lineNumbers index end]) - 1}]
 				for {set i 0} {$i < $e} {incr i} {
 					lappend idx $i
 				}
-			} {
+			} else {
 				set idx [lsearch -all [$sm_lineNumbers get 1.0 end] $lineNum]
 			}
 
@@ -1949,7 +1991,7 @@ class RightPanel {
 			}
 			foreach i $idx {
 				incr i
-				if {[string equal [$sm_text get $i.0 [list $i.0 lineend]] $symbol_name]} {
+				if {[string equal [$sm_text get $i.2 [list $i.0 lineend]] $symbol_name]} {
 					$sm_text delete $i.0 $i.0+1l
 					$sm_lineNumbers delete $i.0 $i.0+1l
 					break
@@ -2080,7 +2122,7 @@ class RightPanel {
 	 # This function takes any list of arguments
 	 # @return void
 	public method rightPanel_configure args {
-		::configDialogs::rightPanel::mkDialog $args
+		::configDialogues::rightPanel::mkDialog $args
 	}
 
 	## Return true if this panel is in visible state
@@ -2092,7 +2134,7 @@ class RightPanel {
 	public method getRightPanelSize {} {
 		if {$PanelVisible} {
 			return $PanelSize
-		} {
+		} else {
 			return $last_PanelSize
 		}
 	}
@@ -2108,7 +2150,7 @@ class RightPanel {
 		if {$PanelVisible} {
 			$parentPane paneconfigure $notebook_frame -minsize 0
 
-			pack forget $notebook		;# Hide notebook
+			pack forget [$notebook get_nb]	;# Hide notebook
 			set last_PanelSize $PanelSize	;# Save current panel width
 			set PanelSize 60		;# Change panel width
 			right_panel_redraw_pane		;# Redraw panel
@@ -2122,7 +2164,7 @@ class RightPanel {
 			set PanelVisible 0
 
 		# Show the panel
-		} {
+		} else {
 			$parentPane paneconfigure $notebook_frame -minsize 295
 
 			# Hide button bar
@@ -2131,7 +2173,7 @@ class RightPanel {
 			set PanelSize $last_PanelSize
 			right_panel_redraw_pane
 			$notebook raise $active_page
-			pack $notebook -expand 1 -fill both
+			pack [$notebook get_nb] -expand 1 -fill both
 			# Show pane sash
 			$parentPane configure -sashwidth 2
 			bind $parentPane <Button> {}
@@ -2180,7 +2222,7 @@ class RightPanel {
 		}
 	}
 
-	## Set panel width acording to current sash position
+	## Set panel width according to current sash position
 	 # @return void
 	public method right_panel_set_size {} {
 		set PanelSize [lindex [$parentPane sash coord 0] 0]
@@ -2214,10 +2256,10 @@ class RightPanel {
 			set size $fontSize
 		} elseif {$use_editor_font == -1} {
 			set font $::DEFAULT_FIXED_FONT
-			set size -14
-		} {
+			set size [expr {int(-14 * $::font_size_factor)}]
+		} else {
 			set font $::DEFAULT_FIXED_FONT
-			set size -12
+			set size [expr {int(-12 * $::font_size_factor)}]
 		}
 
 		# Iterate over tags definition
@@ -2234,7 +2276,7 @@ class RightPanel {
 							-weight {bold}		\
 							-slant {roman}]
 				# Italic font
-				} {
+				} else {
 					$widget tag configure [lindex $tag 0]	\
 						-foreground [lindex $tag 1]	\
 						-font [font create -size $size	\
@@ -2244,7 +2286,7 @@ class RightPanel {
 				}
 
 			# No bold, no italic
-			} {
+			} else {
 				$widget tag configure [lindex $tag 0]	\
 					-foreground [lindex $tag 1]	\
 					-font [font create -size $size	\
@@ -2271,3 +2313,7 @@ class RightPanel {
 		$this right_panel_instruction_details_set_enabled $enabled
 	}
 }
+
+# >>> File inclusion guard
+}
+# <<< File inclusion guard
