@@ -2,7 +2,7 @@
 # Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )
 
 ############################################################################
-#    Copyright (C) 2007-2009 by Martin Ošmera                              #
+#    Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 by Martin Ošmera     #
 #    martin.osmera@gmail.com                                               #
 #                                                                          #
 #    This program is free software; you can redistribute it and#or modify  #
@@ -21,6 +21,11 @@
 #    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
 ############################################################################
 
+# >>> File inclusion guard
+if { ! [ info exists _ENGINE_OPCODES_TCL ] } {
+set _ENGINE_OPCODES_TCL _
+# <<< File inclusion guard
+
 # --------------------------------------------------------------------------
 # DESCRIPTION
 # Part of simulator engine functionality.
@@ -433,3 +438,7 @@ private method 108	{} {ins_xrl 224 $ram([R 4])}			;# 0x6C :: xrl  A, R4
 private method 109	{} {ins_xrl 224 $ram([R 5])}			;# 0x6D :: xrl  A, R5
 private method 110	{} {ins_xrl 224 $ram([R 6])}			;# 0x6E :: xrl  A, R6
 private method 111	{} {ins_xrl 224 $ram([R 7])}			;# 0x6F :: xrl  A, R7
+
+# >>> File inclusion guard
+}
+# <<< File inclusion guard
