@@ -2,7 +2,7 @@
 # Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )
 
 ############################################################################
-#    Copyright (C) 2007-2009 by Martin Ošmera                              #
+#    Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 by Martin Ošmera     #
 #    martin.osmera@gmail.com                                               #
 #                                                                          #
 #    This program is free software; you can redistribute it and#or modify  #
@@ -21,6 +21,11 @@
 #    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
 ############################################################################
 
+# >>> File inclusion guard
+if { ! [ info exists _SYMBOL_VIEWER_TCL ] } {
+set _SYMBOL_VIEWER_TCL _
+# <<< File inclusion guard
+
 # --------------------------------------------------------------------------
 # DESCRIPTION
 # Implements assembly language symbols viewer (from code listing)
@@ -31,17 +36,21 @@ class SymbolViewer {
 	 # Int: Counter of object intances
 	common count	0
 	 # Font: Just normal font used in the table
-	common normal_font	[font create	\
-		-family $::DEFAULT_FIXED_FONT	\
-		-size -15			\
-		-weight normal			\
+	common normal_font	[font create			\
+		-family $::DEFAULT_FIXED_FONT			\
+		-size [expr {int(-14 * $::font_size_factor)}]	\
+		-weight normal					\
 	]
 	 # Font: Bold font (the same size as $normal_font)
-	common bold_font	[font create	\
-		-family $::DEFAULT_FIXED_FONT	\
-		-size -15			\
-		-weight bold			\
+	common bold_font	[font create			\
+		-family $::DEFAULT_FIXED_FONT			\
+		-size [expr {int(-14 * $::font_size_factor)}]	\
+		-weight bold					\
 	]
+	if {$::MICROSOFT_WINDOWS} {
+		# it's better to do not use bold font for this purpose on MS Windows®
+		set bold_font $normal_font
+	}
 	 # Dialog configuration
 	common config_list	$::CONFIG(SYMBOL_VIEWER_CONFIG)
 
@@ -62,7 +71,7 @@ class SymbolViewer {
 
 	constructor {} {
 		# Create dialog window
-		set win [toplevel .symbolviewer$count -class {Defined symbols} -bg {#EEEEEE}]
+		set win [toplevel .symbolviewer$count -class {Defined symbols} -bg ${::COMMON_BG_COLOR}]
 		set obj_idx $count
 		incr count
 
@@ -93,16 +102,19 @@ class SymbolViewer {
 		# Configure dialog window
 		wm iconphoto $win ::ICONS::16::symbol
 		wm title $win [mc "Assembly symbol table - MCU 8051 IDE"]
-		wm minsize $win 620 350
+		wm minsize $win 520 350
 		wm protocol $win WM_DELETE_WINDOW "$this close_window"
+		update
 		catch {
-			wm geometry $win [lindex $config_list $i]
+			wm geometry $win [regsub {^\=?\d+x\d+} [lindex $config_list $i]	\
+				[regsub {\+\d+\+\d+$} [wm geometry $win] {}]	\
+			]
 		}
 
 		# Set ...
 		incr i
 		set ::SymbolViewer::display_${obj_idx}(Special)	[lindex $config_list $i]
-		if {[subst "\$::SymbolViewer::display_${obj_idx}(Special)"] == {}} {
+		if {[subst -nocommands "\$::SymbolViewer::display_${obj_idx}(Special)"] == {}} {
 			set ::SymbolViewer::display_${obj_idx}(Special) 1
 		}
 	}
@@ -114,16 +126,16 @@ class SymbolViewer {
 		}
 		set i 0
 		foreach name {DATA IDATA XDATA CODE BIT Number used unused} {
-			lset config_list $i [subst "\$::SymbolViewer::display_${obj_idx}($name)"]
+			lset config_list $i [subst -nocommands "\$::SymbolViewer::display_${obj_idx}($name)"]
 			incr i
 		}
-		lset config_list $i [subst "\$::SymbolViewer::sort_by_${obj_idx}"]
+		lset config_list $i [subst -nocommands "\$::SymbolViewer::sort_by_${obj_idx}"]
 		incr i
-		lset config_list $i [subst "\$::SymbolViewer::sort_by_order_${obj_idx}"]
+		lset config_list $i [subst -nocommands "\$::SymbolViewer::sort_by_order_${obj_idx}"]
 		incr i
 		lset config_list $i [wm geometry $win]
 		incr i
-		lset config_list $i [subst "\$::SymbolViewer::display_${obj_idx}(Special)"]
+		lset config_list $i [subst -nocommands "\$::SymbolViewer::display_${obj_idx}(Special)"]
 
 		# Clean up
 		unset ::SymbolViewer::sort_by_${obj_idx}
@@ -182,7 +194,7 @@ class SymbolViewer {
 		$text_widget tag configure used_YES	-foreground {#00DD00}
 		$text_widget tag configure used_NO	-foreground {#DD0000}
 		$text_widget tag configure tag_sel	-background {#DDDDDD} -font $bold_font
-		$text_widget tag configure nth_row	-background {#EEEEEE}
+		$text_widget tag configure nth_row	-background ${::COMMON_BG_COLOR}
 
 		$text_widget tag raise tag_sel nth_row
 	}
@@ -383,10 +395,10 @@ class SymbolViewer {
 		if {$opened_file == {}} {
 			if {${::X::project_menu_locked}} {
 				set directory {~}
-			} {
+			} else {
 				set directory [${::X::actualProject} cget -projectPath]
 			}
-		} {
+		} else {
 			set directory [file dirname $opened_file]
 		}
 
@@ -394,10 +406,10 @@ class SymbolViewer {
 		KIFSD::FSD ::fsd	 				\
 			-title [mc "Load symbol table - MCU 8051 IDE"]	\
 			-directory $directory -master $win		\
-			-defaultmask 0 -multiple 0 -filetypes {
-				{{Code listing}		{*.lst}	}
-				{{All files}		{*}	}
-			}
+			-defaultmask 0 -multiple 0 -filetypes [list		\
+				[list [mc "Code listing"]	{*.lst}	]	\
+				[list [mc "All files"]		{*}	]	\
+			]
 
 		# Open the selected after press of OK button
 		::fsd setokcmd "
@@ -420,7 +432,7 @@ class SymbolViewer {
 	public method open_file {ignore_errors filename} {
 		if {[catch {
 			set file [open $filename r]
-		}]} {
+		}]} then {
 			if {!$ignore_errors} {
 				tk_messageBox		\
 					-parent $win	\
@@ -472,7 +484,7 @@ class SymbolViewer {
 				set type [lindex $line 0]
 				if {$type == {S}} {
 					set addr [lindex $line end]
-				} {
+				} else {
 					set addr [string replace [lindex $line 2] end end]
 				}
 
@@ -516,7 +528,7 @@ class SymbolViewer {
 
 			if {$type == {S} || $type == {R}} {
 				lappend symbol_table_data [list $name $type $addr {0} $used]
-			} {
+			} else {
 				lappend symbol_table_data [list $name $type $addr [expr "0x$addr"] $used]
 			}
 		}
@@ -528,18 +540,18 @@ class SymbolViewer {
 		refresh
 	}
 
-	## Sort list of loaded symbols acording to user settings
+	## Sort list of loaded symbols according to user settings
 	 # @return void
 	private method sort_table {} {
-		if {[subst "\$::SymbolViewer::sort_by_order_${obj_idx}"]} {
+		if {[subst -nocommands "\$::SymbolViewer::sort_by_order_${obj_idx}"]} {
 			set order {-decreasing}
-		} {
+		} else {
 			set order {-increasing}
 		}
-		set index [subst "\$::SymbolViewer::sort_by_${obj_idx}"]
+		set index [subst -nocommands "\$::SymbolViewer::sort_by_${obj_idx}"]
 		if {$index == 3} {
 			set type {-integer}
-		} {
+		} else {
 			set type {-dictionary}
 		}
 
@@ -562,21 +574,21 @@ class SymbolViewer {
 			{S} {set type {Special}}
 			{R} {set type {Special}}
 		}
-		if {![subst "\$::SymbolViewer::display_${obj_idx}($type)"]} {
+		if {![subst -nocommands "\$::SymbolViewer::display_${obj_idx}($type)"]} {
 			return 0
 		}
 
 		# Adjust flag USED
 		if {$used} {
-			if {![subst "\$::SymbolViewer::display_${obj_idx}(used)"]} {
+			if {![subst -nocommands "\$::SymbolViewer::display_${obj_idx}(used)"]} {
 				return 0
 			}
-			set used {YES}
-		} {
-			if {![subst "\$::SymbolViewer::display_${obj_idx}(unused)"]} {
+			set used [mc "YES"]
+		} else {
+			if {![subst -nocommands "\$::SymbolViewer::display_${obj_idx}(unused)"]} {
 				return 0
 			}
-			set used {NO}
+			set used [mc "NO"]
 		}
 
 		return [list $type $used]
@@ -602,7 +614,7 @@ class SymbolViewer {
 			lset current_line 1 [string index [lindex $current_line 1] 0]
 			if {[lindex $current_line 4] == {YES}} {
 				lset current_line 4 1
-			} {
+			} else {
 				lset current_line 4 0
 			}
 		}
@@ -646,7 +658,7 @@ class SymbolViewer {
 			if {$type == {Special}} {
 				$text_widget insert insert $hexv
 				$text_widget insert insert [string repeat { } [expr {16 - [string length $hexv]}]]
-			} {
+			} else {
 				$text_widget insert insert $hexv
 				$text_widget insert insert {    }
 				$text_widget insert insert $decv
@@ -673,7 +685,7 @@ class SymbolViewer {
 		if {$cur_found} {
 			$text_widget tag add tag_sel $current_line.0 $current_line.0+1l
 			$text_widget see $current_line.0
-		} {
+		} else {
 			set current_line {}
 		}
 
@@ -706,7 +718,7 @@ class SymbolViewer {
 			set string [string tolower [string trimleft $string 0]]
 			$clear_but configure -state normal
 		# Empty string -> abort
-		} {
+		} else {
 			$search_entry configure -style TEntry
 			$clear_but configure -state disabled
 			return 1
@@ -749,7 +761,7 @@ class SymbolViewer {
 		if {$found_idx == -1} {
 			$search_entry configure -style StringNotFound.TEntry
 		# String found
-		} {
+		} else {
 			set current_line [expr {$found_idx + 1}]
 			$text_widget tag add tag_sel $current_line.0 $current_line.0+1l
 			$text_widget see $current_line.0
@@ -795,7 +807,7 @@ class SymbolViewer {
 		select_line $x $y
 		if {$current_line == {}} {
 			set state disabled
-		} {
+		} else {
 			set state normal
 		}
 		foreach entry {{Copy symbol name} {Copy hex value} {Copy dec value} {Copy line}} {
@@ -835,3 +847,7 @@ class SymbolViewer {
 		clipboard append [string trim [$text_widget get $current_line.$s $current_line.$e]]
 	}
 }
+
+# >>> File inclusion guard
+}
+# <<< File inclusion guard
